#include <stdlib.h>
#include <stdio.h>
#include <netinet/in.h>
#include <packet.h>
#include <plugin.h>
#include <decode.h>

int vlan_decode(struct packet *);
int vlan_dprint(struct layer *, char *, int);

struct proto vlan_p=init_proto_ex("802.1q", vlan_decode, vlan_dprint);
struct proto_req vlan_r[]={
	proto_request("ethernet", __constant_htons(0x8100)),
	null_request()
};


int vlan_dprint(struct layer *l, char *buf, int buflen)
{
	u_int16_t vlan, prio, cfi;

	vlan=ntohs(l->h.vlan->vlan);

	prio=vlan & 0xe000;
	cfi =vlan & 0x2000;
	vlan=vlan & 0x0fff;
	
	return snprintf(buf, buflen, "vlan=%u priority=%u cfi=%u proto=0x%x",
		vlan, prio, cfi,
		ntohs(l->h.vlan->proto));
}

int vlan_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	
	/* Check we aren't overflowing the array */
	if ( p->llen >= PKT_LAYERS ) return 0;
	
	/* Fill in the next layers pointer */
	if ( (p->layer[p->llen].h.raw=
		l->h.raw+sizeof(struct pkt_vlanhdr))
		>= p->end ) return 0;

	/* Find a relevent child */
	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.vlan->proto == pc->id ) {
			p->layer[p->llen].flags=0;
			p->layer[p->llen].session=NULL;
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p);
		}
	}

	/* Just data */
	p->layer[p->llen++].proto=NULL;
	return 0;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	int ok=0;
	proc_decode_add decode_add;

	plugin_check(in, out);
	
	PLUGIN_ID("decode.vlan", "802.1q aka vlan");
	PLUGIN_VERSION(1, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=decode_add(&vlan_p, vlan_r);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
